/*
Copyright (C) 2020-2021 Signal Messenger, LLC.
SPDX-License-Identifier: AGPL-3.0-only
*/


#ifndef SIGNAL_FFI_H_
#define SIGNAL_FFI_H_

/* This file was automatically generated by cbindgen */

#include <stdarg.h>
#include <stdbool.h>
#include <stdint.h>
#include <stdlib.h>

typedef enum {
  SignalCiphertextMessageType_Whisper = 2,
  SignalCiphertextMessageType_PreKey = 3,
  SignalCiphertextMessageType_SenderKey = 4,
  SignalCiphertextMessageType_SenderKeyDistribution = 5,
} SignalCiphertextMessageType;

typedef enum {
  SignalDirection_Sending = 0,
  SignalDirection_Receiving = 1,
} SignalDirection;

typedef enum {
  SignalLogLevel_Error = 1,
  SignalLogLevel_Warn,
  SignalLogLevel_Info,
  SignalLogLevel_Debug,
  SignalLogLevel_Trace,
} SignalLogLevel;

typedef enum {
  SignalErrorCode_UnknownError = 1,
  SignalErrorCode_InvalidState = 2,
  SignalErrorCode_InternalError = 3,
  SignalErrorCode_NullParameter = 4,
  SignalErrorCode_InvalidArgument = 5,
  SignalErrorCode_InvalidType = 6,
  SignalErrorCode_InvalidUtf8String = 7,
  SignalErrorCode_InsufficientOutputSize = 8,
  SignalErrorCode_ProtobufError = 10,
  SignalErrorCode_InvalidCiphertext = 20,
  SignalErrorCode_LegacyCiphertextVersion = 21,
  SignalErrorCode_UnknownCiphertextVersion = 22,
  SignalErrorCode_UnrecognizedMessageVersion = 23,
  SignalErrorCode_InvalidMessage = 30,
  SignalErrorCode_SealedSenderSelfSend = 31,
  SignalErrorCode_InvalidKey = 40,
  SignalErrorCode_InvalidSignature = 41,
  SignalErrorCode_FingerprintIdentifierMismatch = 50,
  SignalErrorCode_FingerprintVersionMismatch = 51,
  SignalErrorCode_FingerprintParsingError = 52,
  SignalErrorCode_UntrustedIdentity = 60,
  SignalErrorCode_InvalidKeyIdentifier = 70,
  SignalErrorCode_SessionNotFound = 80,
  SignalErrorCode_DuplicatedMessage = 90,
  SignalErrorCode_CallbackError = 100,
} SignalErrorCode;

typedef struct SignalAes256GcmSiv SignalAes256GcmSiv;

typedef struct SignalCiphertextMessage SignalCiphertextMessage;

typedef struct SignalFingerprint SignalFingerprint;

typedef struct SignalPreKeyBundle SignalPreKeyBundle;

typedef struct SignalPreKeyRecord SignalPreKeyRecord;

typedef struct SignalPreKeySignalMessage SignalPreKeySignalMessage;

typedef struct SignalPrivateKey SignalPrivateKey;

typedef struct SignalProtocolAddress SignalProtocolAddress;

typedef struct SignalPublicKey SignalPublicKey;

typedef struct SignalSenderCertificate SignalSenderCertificate;

typedef struct SignalSenderKeyDistributionMessage SignalSenderKeyDistributionMessage;

typedef struct SignalSenderKeyMessage SignalSenderKeyMessage;

typedef struct SignalSenderKeyName SignalSenderKeyName;

typedef struct SignalSenderKeyRecord SignalSenderKeyRecord;

typedef struct SignalServerCertificate SignalServerCertificate;

typedef struct SignalSessionRecord SignalSessionRecord;

typedef struct SignalFfiError SignalFfiError;

typedef struct SignalMessage SignalMessage;

typedef struct SignalSignedPreKeyRecord SignalSignedPreKeyRecord;

typedef struct SignalUnidentifiedSenderMessageContent SignalUnidentifiedSenderMessageContent;

typedef int (*SignalLoadSession)(void *store_ctx, SignalSessionRecord **recordp, const SignalProtocolAddress *address, void *ctx);

typedef int (*SignalStoreSession)(void *store_ctx, const SignalProtocolAddress *address, const SignalSessionRecord *record, void *ctx);

typedef struct {
  void *ctx;
  SignalLoadSession load_session;
  SignalStoreSession store_session;
} SignalSessionStore;

typedef int (*SignalGetIdentityKeyPair)(void *store_ctx, SignalPrivateKey **keyp, void *ctx);

typedef int (*SignalGetLocalRegistrationId)(void *store_ctx, uint32_t *idp, void *ctx);

typedef int (*SignalSaveIdentityKey)(void *store_ctx, const SignalProtocolAddress *address, const SignalPublicKey *public_key, void *ctx);

typedef int (*SignalGetIdentityKey)(void *store_ctx, SignalPublicKey **public_keyp, const SignalProtocolAddress *address, void *ctx);

typedef int (*SignalIsTrustedIdentity)(void *store_ctx, const SignalProtocolAddress *address, const SignalPublicKey *public_key, unsigned int direction, void *ctx);

typedef struct {
  void *ctx;
  SignalGetIdentityKeyPair get_identity_key_pair;
  SignalGetLocalRegistrationId get_local_registration_id;
  SignalSaveIdentityKey save_identity;
  SignalGetIdentityKey get_identity;
  SignalIsTrustedIdentity is_trusted_identity;
} SignalIdentityKeyStore;

typedef int (*SignalLoadPreKey)(void *store_ctx, SignalPreKeyRecord **recordp, uint32_t id, void *ctx);

typedef int (*SignalStorePreKey)(void *store_ctx, uint32_t id, const SignalPreKeyRecord *record, void *ctx);

typedef int (*SignalRemovePreKey)(void *store_ctx, uint32_t id, void *ctx);

typedef struct {
  void *ctx;
  SignalLoadPreKey load_pre_key;
  SignalStorePreKey store_pre_key;
  SignalRemovePreKey remove_pre_key;
} SignalPreKeyStore;

typedef int (*SignalLoadSignedPreKey)(void *store_ctx, SignalSignedPreKeyRecord **recordp, uint32_t id, void *ctx);

typedef int (*SignalStoreSignedPreKey)(void *store_ctx, uint32_t id, const SignalSignedPreKeyRecord *record, void *ctx);

typedef struct {
  void *ctx;
  SignalLoadSignedPreKey load_signed_pre_key;
  SignalStoreSignedPreKey store_signed_pre_key;
} SignalSignedPreKeyStore;

typedef int (*SignalLoadSenderKey)(void *store_ctx, SignalSenderKeyRecord**, const SignalSenderKeyName*, void *ctx);

typedef int (*SignalStoreSenderKey)(void *store_ctx, const SignalSenderKeyName*, const SignalSenderKeyRecord*, void *ctx);

typedef struct {
  void *ctx;
  SignalLoadSenderKey load_sender_key;
  SignalStoreSenderKey store_sender_key;
} SignalSenderKeyStore;

typedef bool (*SignalLogEnabledCallback)(const char *target, SignalLogLevel level);

typedef void (*SignalLogCallback)(const char *target, SignalLogLevel level, const char *file, uint32_t line, const char *message);

typedef void (*SignalLogFlushCallback)(void);

typedef struct {
  SignalLogEnabledCallback enabled;
  SignalLogCallback log;
  SignalLogFlushCallback flush;
} SignalFfiLogger;

void signal_print_ptr(const void *p);

void signal_free_string(const char *buf);

void signal_free_buffer(const unsigned char *buf, size_t buf_len);

SignalFfiError *signal_error_get_message(const SignalFfiError *err, const char **out);

uint32_t signal_error_get_type(const SignalFfiError *err);

void signal_error_free(SignalFfiError *err);

SignalFfiError *signal_hkdf_derive(unsigned char *output,
                                   size_t output_length,
                                   int version,
                                   const unsigned char *input_key_material,
                                   size_t input_key_material_len,
                                   const unsigned char *salt,
                                   size_t salt_len,
                                   const unsigned char *info,
                                   size_t info_len);

SignalFfiError *signal_address_get_device_id(const SignalProtocolAddress *obj, unsigned int *out);

SignalFfiError *signal_address_clone(SignalProtocolAddress **new_obj,
                                     const SignalProtocolAddress *obj);

SignalFfiError *signal_publickey_clone(SignalPublicKey **new_obj, const SignalPublicKey *obj);

SignalFfiError *signal_privatekey_clone(SignalPrivateKey **new_obj, const SignalPrivateKey *obj);

SignalFfiError *signal_identitykeypair_deserialize(SignalPrivateKey **private_key,
                                                   SignalPublicKey **public_key,
                                                   const unsigned char *input,
                                                   size_t input_len);

SignalFfiError *signal_session_record_get_remote_registration_id(const SignalSessionRecord *obj,
                                                                 unsigned int *out);

SignalFfiError *signal_session_record_archive_current_state(SignalSessionRecord *session_record);

SignalFfiError *signal_session_record_has_current_state(bool *result,
                                                        const SignalSessionRecord *session_record);

SignalFfiError *signal_session_record_clone(SignalSessionRecord **new_obj,
                                            const SignalSessionRecord *obj);

SignalFfiError *signal_fingerprint_new(SignalFingerprint **obj,
                                       unsigned int iterations,
                                       unsigned int version,
                                       const unsigned char *local_identifier,
                                       size_t local_identifier_len,
                                       const SignalPublicKey *local_key,
                                       const unsigned char *remote_identifier,
                                       size_t remote_identifier_len,
                                       const SignalPublicKey *remote_key);

SignalFfiError *signal_fingerprint_clone(SignalFingerprint **new_obj, const SignalFingerprint *obj);

SignalFfiError *signal_message_clone(SignalMessage **new_obj, const SignalMessage *obj);

SignalFfiError *signal_message_get_sender_ratchet_key(SignalPublicKey **new_obj,
                                                      const SignalMessage *obj);

SignalFfiError *signal_message_get_message_version(const SignalMessage *obj, unsigned int *out);

SignalFfiError *signal_message_get_counter(const SignalMessage *obj, unsigned int *out);

SignalFfiError *signal_pre_key_signal_message_clone(SignalPreKeySignalMessage **new_obj,
                                                    const SignalPreKeySignalMessage *obj);

SignalFfiError *signal_pre_key_signal_message_get_version(const SignalPreKeySignalMessage *obj,
                                                          unsigned int *out);

SignalFfiError *signal_pre_key_signal_message_get_registration_id(const SignalPreKeySignalMessage *obj,
                                                                  unsigned int *out);

SignalFfiError *signal_pre_key_signal_message_get_pre_key_id(const SignalPreKeySignalMessage *obj,
                                                             unsigned int *out);

SignalFfiError *signal_pre_key_signal_message_get_signed_pre_key_id(const SignalPreKeySignalMessage *obj,
                                                                    unsigned int *out);

SignalFfiError *signal_pre_key_signal_message_get_base_key(SignalPublicKey **new_obj,
                                                           const SignalPreKeySignalMessage *obj);

SignalFfiError *signal_pre_key_signal_message_get_identity_key(SignalPublicKey **new_obj,
                                                               const SignalPreKeySignalMessage *obj);

SignalFfiError *signal_pre_key_signal_message_get_signal_message(SignalMessage **new_obj,
                                                                 const SignalPreKeySignalMessage *obj);

SignalFfiError *signal_sender_key_message_clone(SignalSenderKeyMessage **new_obj,
                                                const SignalSenderKeyMessage *obj);

SignalFfiError *signal_sender_key_message_get_key_id(const SignalSenderKeyMessage *obj,
                                                     unsigned int *out);

SignalFfiError *signal_sender_key_message_get_iteration(const SignalSenderKeyMessage *obj,
                                                        unsigned int *out);

SignalFfiError *signal_sender_key_distribution_message_clone(SignalSenderKeyDistributionMessage **new_obj,
                                                             const SignalSenderKeyDistributionMessage *obj);

SignalFfiError *signal_sender_key_distribution_message_get_id(const SignalSenderKeyDistributionMessage *obj,
                                                              unsigned int *out);

SignalFfiError *signal_sender_key_distribution_message_get_iteration(const SignalSenderKeyDistributionMessage *obj,
                                                                     unsigned int *out);

SignalFfiError *signal_sender_key_distribution_message_get_signature_key(SignalPublicKey **new_obj,
                                                                         const SignalSenderKeyDistributionMessage *obj);

SignalFfiError *signal_pre_key_bundle_new(SignalPreKeyBundle **obj,
                                          unsigned int registration_id,
                                          unsigned int device_id,
                                          const unsigned int *prekey_id,
                                          const SignalPublicKey *prekey,
                                          unsigned int signed_prekey_id,
                                          const SignalPublicKey *signed_prekey,
                                          const unsigned char *signed_prekey_signature,
                                          size_t signed_prekey_signature_len,
                                          const SignalPublicKey *identity_key);

SignalFfiError *signal_pre_key_bundle_clone(SignalPreKeyBundle **new_obj,
                                            const SignalPreKeyBundle *obj);

SignalFfiError *signal_pre_key_bundle_get_registration_id(const SignalPreKeyBundle *obj,
                                                          unsigned int *out);

SignalFfiError *signal_pre_key_bundle_get_device_id(const SignalPreKeyBundle *obj,
                                                    unsigned int *out);

SignalFfiError *signal_pre_key_bundle_get_signed_pre_key_id(const SignalPreKeyBundle *obj,
                                                            unsigned int *out);

SignalFfiError *signal_pre_key_bundle_get_pre_key_id(const SignalPreKeyBundle *obj,
                                                     unsigned int *out);

SignalFfiError *signal_pre_key_bundle_get_pre_key_public(SignalPublicKey **new_obj,
                                                         const SignalPreKeyBundle *obj);

SignalFfiError *signal_pre_key_bundle_get_signed_pre_key_public(SignalPublicKey **new_obj,
                                                                const SignalPreKeyBundle *obj);

SignalFfiError *signal_pre_key_bundle_get_identity_key(SignalPublicKey **new_obj,
                                                       const SignalPreKeyBundle *obj);

SignalFfiError *signal_signed_pre_key_record_get_id(const SignalSignedPreKeyRecord *obj,
                                                    unsigned int *out);

SignalFfiError *signal_signed_pre_key_record_get_timestamp(const SignalSignedPreKeyRecord *obj,
                                                           unsigned long long *out);

SignalFfiError *signal_signed_pre_key_record_get_public_key(SignalPublicKey **new_obj,
                                                            const SignalSignedPreKeyRecord *obj);

SignalFfiError *signal_signed_pre_key_record_get_private_key(SignalPrivateKey **new_obj,
                                                             const SignalSignedPreKeyRecord *obj);

SignalFfiError *signal_signed_pre_key_record_clone(SignalSignedPreKeyRecord **new_obj,
                                                   const SignalSignedPreKeyRecord *obj);

SignalFfiError *signal_pre_key_record_get_id(const SignalPreKeyRecord *obj, unsigned int *out);

SignalFfiError *signal_pre_key_record_get_public_key(SignalPublicKey **new_obj,
                                                     const SignalPreKeyRecord *obj);

SignalFfiError *signal_pre_key_record_get_private_key(SignalPrivateKey **new_obj,
                                                      const SignalPreKeyRecord *obj);

SignalFfiError *signal_pre_key_record_clone(SignalPreKeyRecord **new_obj,
                                            const SignalPreKeyRecord *obj);

SignalFfiError *signal_sender_key_name_clone(SignalSenderKeyName **new_obj,
                                             const SignalSenderKeyName *obj);

SignalFfiError *signal_sender_key_name_get_sender_device_id(const SignalSenderKeyName *obj,
                                                            unsigned int *out);

SignalFfiError *signal_sender_key_record_clone(SignalSenderKeyRecord **new_obj,
                                               const SignalSenderKeyRecord *obj);

SignalFfiError *signal_process_prekey_bundle(SignalPreKeyBundle *bundle,
                                             const SignalProtocolAddress *protocol_address,
                                             const SignalSessionStore *session_store,
                                             const SignalIdentityKeyStore *identity_key_store,
                                             void *ctx);

SignalFfiError *signal_encrypt_message(SignalCiphertextMessage **msg,
                                       const unsigned char *ptext,
                                       size_t ptext_len,
                                       const SignalProtocolAddress *protocol_address,
                                       const SignalSessionStore *session_store,
                                       const SignalIdentityKeyStore *identity_key_store,
                                       void *ctx);

SignalFfiError *signal_ciphertext_message_type(uint8_t *typ, const SignalCiphertextMessage *msg);

SignalFfiError *signal_ciphertext_message_serialize(const unsigned char **result,
                                                    size_t *result_len,
                                                    const SignalCiphertextMessage *msg);

SignalFfiError *signal_decrypt_message(const unsigned char **result,
                                       size_t *result_len,
                                       const SignalMessage *message,
                                       const SignalProtocolAddress *protocol_address,
                                       const SignalSessionStore *session_store,
                                       const SignalIdentityKeyStore *identity_key_store,
                                       void *ctx);

SignalFfiError *signal_decrypt_pre_key_message(const unsigned char **result,
                                               size_t *result_len,
                                               const SignalPreKeySignalMessage *message,
                                               const SignalProtocolAddress *protocol_address,
                                               const SignalSessionStore *session_store,
                                               const SignalIdentityKeyStore *identity_key_store,
                                               const SignalPreKeyStore *prekey_store,
                                               const SignalSignedPreKeyStore *signed_prekey_store,
                                               void *ctx);

SignalFfiError *signal_create_sender_key_distribution_message(SignalSenderKeyDistributionMessage **obj,
                                                              const SignalSenderKeyName *sender_key_name,
                                                              const SignalSenderKeyStore *store,
                                                              void *ctx);

SignalFfiError *signal_process_sender_key_distribution_message(const SignalSenderKeyName *sender_key_name,
                                                               const SignalSenderKeyDistributionMessage *sender_key_distribution_message,
                                                               const SignalSenderKeyStore *store,
                                                               void *ctx);

SignalFfiError *signal_group_encrypt_message(const unsigned char **out,
                                             size_t *out_len,
                                             const SignalSenderKeyName *sender_key_name,
                                             const unsigned char *message,
                                             size_t message_len,
                                             const SignalSenderKeyStore *store,
                                             void *ctx);

SignalFfiError *signal_group_decrypt_message(const unsigned char **out,
                                             size_t *out_len,
                                             const SignalSenderKeyName *sender_key_name,
                                             const unsigned char *message,
                                             size_t message_len,
                                             const SignalSenderKeyStore *store,
                                             void *ctx);

SignalFfiError *signal_server_certificate_get_key_id(const SignalServerCertificate *obj,
                                                     unsigned int *out);

SignalFfiError *signal_server_certificate_get_key(SignalPublicKey **new_obj,
                                                  const SignalServerCertificate *obj);

SignalFfiError *signal_sender_certificate_get_expiration(const SignalSenderCertificate *obj,
                                                         unsigned long long *out);

SignalFfiError *signal_sender_certificate_get_device_id(const SignalSenderCertificate *obj,
                                                        unsigned int *out);

SignalFfiError *signal_sender_certificate_get_key(SignalPublicKey **new_obj,
                                                  const SignalSenderCertificate *obj);

SignalFfiError *signal_sender_certificate_get_server_certificate(SignalServerCertificate **new_obj,
                                                                 const SignalSenderCertificate *obj);

SignalFfiError *signal_unidentified_sender_message_content_get_msg_type(uint8_t *out,
                                                                        const SignalUnidentifiedSenderMessageContent *obj);

SignalFfiError *signal_unidentified_sender_message_content_get_sender_cert(SignalSenderCertificate **new_obj,
                                                                           const SignalUnidentifiedSenderMessageContent *obj);

SignalFfiError *signal_sealed_session_cipher_encrypt(const unsigned char **out,
                                                     size_t *out_len,
                                                     const SignalProtocolAddress *destination,
                                                     const SignalSenderCertificate *sender_cert,
                                                     const unsigned char *ptext,
                                                     size_t ptext_len,
                                                     const SignalSessionStore *session_store,
                                                     const SignalIdentityKeyStore *identity_key_store,
                                                     void *ctx);

SignalFfiError *signal_sealed_session_cipher_decrypt_to_usmc(SignalUnidentifiedSenderMessageContent **out,
                                                             const unsigned char *ctext,
                                                             size_t ctext_len,
                                                             const SignalIdentityKeyStore *identity_store,
                                                             void *ctx);

SignalFfiError *signal_sealed_session_cipher_decrypt(const unsigned char **out,
                                                     size_t *out_len,
                                                     const char **sender_e164,
                                                     const char **sender_uuid,
                                                     uint32_t *sender_device_id,
                                                     const unsigned char *ctext,
                                                     size_t ctext_len,
                                                     const SignalPublicKey *trust_root,
                                                     uint64_t timestamp,
                                                     const char *local_e164,
                                                     const char *local_uuid,
                                                     unsigned int local_device_id,
                                                     const SignalSessionStore *session_store,
                                                     const SignalIdentityKeyStore *identity_store,
                                                     const SignalPreKeyStore *prekey_store,
                                                     const SignalSignedPreKeyStore *signed_prekey_store,
                                                     void *ctx);

void signal_init_logger(SignalLogLevel max_level, SignalFfiLogger logger);

SignalFfiError *signal_address_destroy(SignalProtocolAddress *p);

SignalFfiError *signal_address_get_name(const SignalProtocolAddress *obj, const char **out);

SignalFfiError *signal_address_new(SignalProtocolAddress **out,
                                   const char *name,
                                   uint32_t device_id);

SignalFfiError *signal_publickey_destroy(SignalPublicKey *p);

SignalFfiError *signal_publickey_deserialize(SignalPublicKey **p,
                                             const unsigned char *data,
                                             size_t data_len);

SignalFfiError *signal_publickey_serialize(const SignalPublicKey *obj,
                                           const unsigned char **out,
                                           size_t *out_len);

SignalFfiError *signal_publickey_get_public_key_bytes(const SignalPublicKey *obj,
                                                      const unsigned char **out,
                                                      size_t *out_len);

SignalFfiError *signal_publickey_compare(int32_t *out,
                                         const SignalPublicKey *key1,
                                         const SignalPublicKey *key2);

SignalFfiError *signal_publickey_verify(bool *out,
                                        const SignalPublicKey *key,
                                        const unsigned char *message,
                                        size_t message_len,
                                        const unsigned char *signature,
                                        size_t signature_len);

SignalFfiError *signal_privatekey_destroy(SignalPrivateKey *p);

SignalFfiError *signal_privatekey_deserialize(SignalPrivateKey **p,
                                              const unsigned char *data,
                                              size_t data_len);

SignalFfiError *signal_privatekey_serialize(const SignalPrivateKey *obj,
                                            const unsigned char **out,
                                            size_t *out_len);

SignalFfiError *signal_privatekey_generate(SignalPrivateKey **out);

SignalFfiError *signal_privatekey_get_public_key(SignalPublicKey **out, const SignalPrivateKey *k);

SignalFfiError *signal_privatekey_sign(const unsigned char **out,
                                       size_t *out_len,
                                       const SignalPrivateKey *key,
                                       const unsigned char *message,
                                       size_t message_len);

SignalFfiError *signal_privatekey_agree(const unsigned char **out,
                                        size_t *out_len,
                                        const SignalPrivateKey *private_key,
                                        const SignalPublicKey *public_key);

SignalFfiError *signal_identitykeypair_serialize(const unsigned char **out,
                                                 size_t *out_len,
                                                 const SignalPublicKey *public_key,
                                                 const SignalPrivateKey *private_key);

SignalFfiError *signal_fingerprint_destroy(SignalFingerprint *p);

SignalFfiError *signal_fingerprint_scannable_encoding(const SignalFingerprint *obj,
                                                      const unsigned char **out,
                                                      size_t *out_len);

SignalFfiError *signal_fingerprint_display_string(const SignalFingerprint *obj, const char **out);

SignalFfiError *signal_fingerprint_format(const char **out,
                                          const unsigned char *local,
                                          size_t local_len,
                                          const unsigned char *remote,
                                          size_t remote_len);

SignalFfiError *signal_fingerprint_compare(bool *out,
                                           const unsigned char *fprint1,
                                           size_t fprint1_len,
                                           const unsigned char *fprint2,
                                           size_t fprint2_len);

SignalFfiError *signal_message_destroy(SignalMessage *p);

SignalFfiError *signal_message_deserialize(SignalMessage **p,
                                           const unsigned char *data,
                                           size_t data_len);

SignalFfiError *signal_message_get_body(const SignalMessage *obj,
                                        const unsigned char **out,
                                        size_t *out_len);

SignalFfiError *signal_message_get_serialized(const SignalMessage *obj,
                                              const unsigned char **out,
                                              size_t *out_len);

SignalFfiError *signal_message_new(SignalMessage **out,
                                   uint8_t message_version,
                                   const unsigned char *mac_key,
                                   size_t mac_key_len,
                                   const SignalPublicKey *sender_ratchet_key,
                                   uint32_t counter,
                                   uint32_t previous_counter,
                                   const unsigned char *ciphertext,
                                   size_t ciphertext_len,
                                   const SignalPublicKey *sender_identity_key,
                                   const SignalPublicKey *receiver_identity_key);

SignalFfiError *signal_message_verify_mac(bool *out,
                                          const SignalMessage *msg,
                                          const SignalPublicKey *sender_identity_key,
                                          const SignalPublicKey *receiver_identity_key,
                                          const unsigned char *mac_key,
                                          size_t mac_key_len);

SignalFfiError *signal_pre_key_signal_message_new(SignalPreKeySignalMessage **out,
                                                  uint8_t message_version,
                                                  uint32_t registration_id,
                                                  uint32_t pre_key_id,
                                                  uint32_t signed_pre_key_id,
                                                  const SignalPublicKey *base_key,
                                                  const SignalPublicKey *identity_key,
                                                  const SignalMessage *signal_message);

SignalFfiError *signal_pre_key_signal_message_destroy(SignalPreKeySignalMessage *p);

SignalFfiError *signal_pre_key_signal_message_deserialize(SignalPreKeySignalMessage **p,
                                                          const unsigned char *data,
                                                          size_t data_len);

SignalFfiError *signal_pre_key_signal_message_serialize(const SignalPreKeySignalMessage *obj,
                                                        const unsigned char **out,
                                                        size_t *out_len);

SignalFfiError *signal_sender_key_message_destroy(SignalSenderKeyMessage *p);

SignalFfiError *signal_sender_key_message_deserialize(SignalSenderKeyMessage **p,
                                                      const unsigned char *data,
                                                      size_t data_len);

SignalFfiError *signal_sender_key_message_get_cipher_text(const SignalSenderKeyMessage *obj,
                                                          const unsigned char **out,
                                                          size_t *out_len);

SignalFfiError *signal_sender_key_message_serialize(const SignalSenderKeyMessage *obj,
                                                    const unsigned char **out,
                                                    size_t *out_len);

SignalFfiError *signal_sender_key_message_new(SignalSenderKeyMessage **out,
                                              uint32_t key_id,
                                              uint32_t iteration,
                                              const unsigned char *ciphertext,
                                              size_t ciphertext_len,
                                              const SignalPrivateKey *pk);

SignalFfiError *signal_sender_key_message_verify_signature(bool *out,
                                                           const SignalSenderKeyMessage *skm,
                                                           const SignalPublicKey *pubkey);

SignalFfiError *signal_sender_key_distribution_message_destroy(SignalSenderKeyDistributionMessage *p);

SignalFfiError *signal_sender_key_distribution_message_deserialize(SignalSenderKeyDistributionMessage **p,
                                                                   const unsigned char *data,
                                                                   size_t data_len);

SignalFfiError *signal_sender_key_distribution_message_get_chain_key(const SignalSenderKeyDistributionMessage *obj,
                                                                     const unsigned char **out,
                                                                     size_t *out_len);

SignalFfiError *signal_sender_key_distribution_message_serialize(const SignalSenderKeyDistributionMessage *obj,
                                                                 const unsigned char **out,
                                                                 size_t *out_len);

SignalFfiError *signal_sender_key_distribution_message_new(SignalSenderKeyDistributionMessage **out,
                                                           uint32_t key_id,
                                                           uint32_t iteration,
                                                           const unsigned char *chainkey,
                                                           size_t chainkey_len,
                                                           const SignalPublicKey *pk);

SignalFfiError *signal_pre_key_bundle_destroy(SignalPreKeyBundle *p);

SignalFfiError *signal_pre_key_bundle_get_signed_pre_key_signature(const SignalPreKeyBundle *obj,
                                                                   const unsigned char **out,
                                                                   size_t *out_len);

SignalFfiError *signal_signed_pre_key_record_destroy(SignalSignedPreKeyRecord *p);

SignalFfiError *signal_signed_pre_key_record_deserialize(SignalSignedPreKeyRecord **p,
                                                         const unsigned char *data,
                                                         size_t data_len);

SignalFfiError *signal_signed_pre_key_record_get_signature(const SignalSignedPreKeyRecord *obj,
                                                           const unsigned char **out,
                                                           size_t *out_len);

SignalFfiError *signal_signed_pre_key_record_serialize(const SignalSignedPreKeyRecord *obj,
                                                       const unsigned char **out,
                                                       size_t *out_len);

SignalFfiError *signal_signed_pre_key_record_new(SignalSignedPreKeyRecord **out,
                                                 uint32_t id,
                                                 uint64_t timestamp,
                                                 const SignalPublicKey *pub_key,
                                                 const SignalPrivateKey *priv_key,
                                                 const unsigned char *signature,
                                                 size_t signature_len);

SignalFfiError *signal_pre_key_record_destroy(SignalPreKeyRecord *p);

SignalFfiError *signal_pre_key_record_deserialize(SignalPreKeyRecord **p,
                                                  const unsigned char *data,
                                                  size_t data_len);

SignalFfiError *signal_pre_key_record_serialize(const SignalPreKeyRecord *obj,
                                                const unsigned char **out,
                                                size_t *out_len);

SignalFfiError *signal_pre_key_record_new(SignalPreKeyRecord **out,
                                          uint32_t id,
                                          const SignalPublicKey *pub_key,
                                          const SignalPrivateKey *priv_key);

SignalFfiError *signal_sender_key_name_destroy(SignalSenderKeyName *p);

SignalFfiError *signal_sender_key_name_get_group_id(const SignalSenderKeyName *obj,
                                                    const char **out);

SignalFfiError *signal_sender_key_name_get_sender_name(const SignalSenderKeyName *obj,
                                                       const char **out);

SignalFfiError *signal_sender_key_name_new(SignalSenderKeyName **out,
                                           const char *group_id,
                                           const char *sender_name,
                                           uint32_t sender_device_id);

SignalFfiError *signal_sender_key_record_destroy(SignalSenderKeyRecord *p);

SignalFfiError *signal_sender_key_record_deserialize(SignalSenderKeyRecord **p,
                                                     const unsigned char *data,
                                                     size_t data_len);

SignalFfiError *signal_sender_key_record_serialize(const SignalSenderKeyRecord *obj,
                                                   const unsigned char **out,
                                                   size_t *out_len);

SignalFfiError *signal_sender_key_record_new_fresh(SignalSenderKeyRecord **out);

SignalFfiError *signal_ciphertext_message_destroy(SignalCiphertextMessage *p);

SignalFfiError *signal_server_certificate_destroy(SignalServerCertificate *p);

SignalFfiError *signal_server_certificate_deserialize(SignalServerCertificate **p,
                                                      const unsigned char *data,
                                                      size_t data_len);

SignalFfiError *signal_server_certificate_get_serialized(const SignalServerCertificate *obj,
                                                         const unsigned char **out,
                                                         size_t *out_len);

SignalFfiError *signal_server_certificate_get_certificate(const SignalServerCertificate *obj,
                                                          const unsigned char **out,
                                                          size_t *out_len);

SignalFfiError *signal_server_certificate_get_signature(const SignalServerCertificate *obj,
                                                        const unsigned char **out,
                                                        size_t *out_len);

SignalFfiError *signal_server_certificate_new(SignalServerCertificate **out,
                                              uint32_t key_id,
                                              const SignalPublicKey *server_key,
                                              const SignalPrivateKey *trust_root);

SignalFfiError *signal_sender_certificate_destroy(SignalSenderCertificate *p);

SignalFfiError *signal_sender_certificate_deserialize(SignalSenderCertificate **p,
                                                      const unsigned char *data,
                                                      size_t data_len);

SignalFfiError *signal_sender_certificate_get_serialized(const SignalSenderCertificate *obj,
                                                         const unsigned char **out,
                                                         size_t *out_len);

SignalFfiError *signal_sender_certificate_get_certificate(const SignalSenderCertificate *obj,
                                                          const unsigned char **out,
                                                          size_t *out_len);

SignalFfiError *signal_sender_certificate_get_signature(const SignalSenderCertificate *obj,
                                                        const unsigned char **out,
                                                        size_t *out_len);

SignalFfiError *signal_sender_certificate_get_sender_uuid(const SignalSenderCertificate *obj,
                                                          const char **out);

SignalFfiError *signal_sender_certificate_get_sender_e164(const SignalSenderCertificate *obj,
                                                          const char **out);

SignalFfiError *signal_sender_certificate_validate(bool *out,
                                                   const SignalSenderCertificate *cert,
                                                   const SignalPublicKey *key,
                                                   uint64_t time);

SignalFfiError *signal_sender_certificate_new(SignalSenderCertificate **out,
                                              const char *sender_uuid,
                                              const char *sender_e164,
                                              uint32_t sender_device_id,
                                              const SignalPublicKey *sender_key,
                                              uint64_t expiration,
                                              const SignalServerCertificate *signer_cert,
                                              const SignalPrivateKey *signer_key);

SignalFfiError *signal_unidentified_sender_message_content_destroy(SignalUnidentifiedSenderMessageContent *p);

SignalFfiError *signal_unidentified_sender_message_content_deserialize(SignalUnidentifiedSenderMessageContent **p,
                                                                       const unsigned char *data,
                                                                       size_t data_len);

SignalFfiError *signal_unidentified_sender_message_content_serialize(const SignalUnidentifiedSenderMessageContent *obj,
                                                                     const unsigned char **out,
                                                                     size_t *out_len);

SignalFfiError *signal_unidentified_sender_message_content_get_contents(const SignalUnidentifiedSenderMessageContent *obj,
                                                                        const unsigned char **out,
                                                                        size_t *out_len);

SignalFfiError *signal_session_record_destroy(SignalSessionRecord *p);

SignalFfiError *signal_session_record_deserialize(SignalSessionRecord **p,
                                                  const unsigned char *data,
                                                  size_t data_len);

SignalFfiError *signal_session_record_serialize(const SignalSessionRecord *obj,
                                                const unsigned char **out,
                                                size_t *out_len);

SignalFfiError *signal_aes256_gcm_siv_destroy(SignalAes256GcmSiv *p);

SignalFfiError *signal_aes256_gcm_siv_new(SignalAes256GcmSiv **out,
                                          const unsigned char *key,
                                          size_t key_len);

SignalFfiError *signal_aes256_gcm_siv_encrypt(const unsigned char **out,
                                              size_t *out_len,
                                              const SignalAes256GcmSiv *aes_gcm_siv,
                                              const unsigned char *ptext,
                                              size_t ptext_len,
                                              const unsigned char *nonce,
                                              size_t nonce_len,
                                              const unsigned char *associated_data,
                                              size_t associated_data_len);

SignalFfiError *signal_aes256_gcm_siv_decrypt(const unsigned char **out,
                                              size_t *out_len,
                                              const SignalAes256GcmSiv *aes_gcm_siv,
                                              const unsigned char *ctext,
                                              size_t ctext_len,
                                              const unsigned char *nonce,
                                              size_t nonce_len,
                                              const unsigned char *associated_data,
                                              size_t associated_data_len);

#endif /* SIGNAL_FFI_H_ */
